@@ -1,16 +1,39 @@
+/*
+ *  This file is part of GNOME Twitch - 'Enjoy Twitch on your GNU/Linux desktop'
+ *  Copyright © 2017 Vincent Szolnoky <vinszent@vinszent.com>
+ *
+ *  GNOME Twitch is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ *
+ *  GNOME Twitch is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with GNOME Twitch. If not, see <http://www.gnu.org/licenses/>.
+ */
+
 #include "gt-games-container-child.h"
+#include "utils.h"
+#include <glib/gi18n.h>
+#include <glib/gprintf.h>
 
 #define TAG "GtGamesContainerChild"
-#include "utils.h"
+#include "gnome-twitch/gt-log.h"
 
 typedef struct
 {
     GtGame* game;
 
-    GtkWidget* preview_image;
-    GtkWidget* middle_revealer;
+    GtkWidget* cover_stack;
+    GtkWidget* cover_image;
+    GtkWidget* cover_overlay_revealer;
     GtkWidget* name_label;
-    GtkWidget* event_box;
+    GtkWidget* viewers_label;
+    GtkWidget* viewers_image;
 } GtGamesContainerChildPrivate;
 
 G_DEFINE_TYPE_WITH_PRIVATE(GtGamesContainerChild, gt_games_container_child, GTK_TYPE_FLOW_BOX_CHILD)
@@ -24,34 +47,75 @@ enum
 
 static GParamSpec* props[NUM_PROPS];
 
-GtGamesContainerChild*
-gt_games_container_child_new(GtGame* game)
+static void
+update_viewers_cb(GObject* source,
+    GParamSpec* pspec, gpointer udata)
 {
-    return g_object_new(GT_TYPE_GAMES_VIEW_CHILD,
-                        "game", game,
-                        NULL);
+    RETURN_IF_FAIL(GT_IS_GAMES_CONTAINER_CHILD(udata));
+    RETURN_IF_FAIL(GT_IS_GAME(source));
+
+    GtGamesContainerChild* self = GT_GAMES_CONTAINER_CHILD(udata);
+    GtGamesContainerChildPrivate* priv = gt_games_container_child_get_instance_private(self);
+
+    gint64 viewers = gt_game_get_viewers(priv->game);
+    gchar label[50];
+
+    if (viewers > 1e4)
+    {
+        // Translators: Used for when viewers >= 1000
+        // Shorthand for thousands. Ex (English): 6200 = 6.2k
+        g_sprintf(label, _("%3.1fk"), (gdouble) viewers / 1e3);
+    }
+    else
+    {
+        gchar num[50];
+
+        g_sprintf(num, "%" G_GINT64_FORMAT, viewers);
+
+        // Translators: Used for when viewers < 1000
+        g_sprintf(label, _("%s"), num);
+
+    }
+
+    gtk_label_set_text(GTK_LABEL(priv->viewers_label), label);
+
+    gtk_widget_set_visible(priv->viewers_label, viewers > -1);
+    gtk_widget_set_visible(priv->viewers_image, viewers > -1);
 }
 
 static void
 motion_enter_cb(GtkWidget* widget,
-                GdkEvent* evt,
-                gpointer udata)
+    GdkEvent* evt, gpointer udata)
 {
     GtGamesContainerChild* self = GT_GAMES_CONTAINER_CHILD(udata);
     GtGamesContainerChildPrivate* priv = gt_games_container_child_get_instance_private(self);
 
-    gtk_revealer_set_reveal_child(GTK_REVEALER(priv->middle_revealer), TRUE);
+    gtk_revealer_set_reveal_child(GTK_REVEALER(priv->cover_overlay_revealer), TRUE);
 }
 
 static void
 motion_leave_cb(GtkWidget* widget,
-                GdkEvent* evt,
-                gpointer udata)
+    GdkEvent* evt, gpointer udata)
 {
     GtGamesContainerChild* self = GT_GAMES_CONTAINER_CHILD(udata);
     GtGamesContainerChildPrivate* priv = gt_games_container_child_get_instance_private(self);
 
-    gtk_revealer_set_reveal_child(GTK_REVEALER(priv->middle_revealer), FALSE);
+    gtk_revealer_set_reveal_child(GTK_REVEALER(priv->cover_overlay_revealer), FALSE);
+}
+
+static void
+updating_cb(GObject* source,
+    GParamSpec* pspec, gpointer udata)
+{
+    g_assert(GT_IS_GAMES_CONTAINER_CHILD(udata));
+    g_assert(GT_IS_GAME(source));
+
+    GtGamesContainerChild* self = GT_GAMES_CONTAINER_CHILD(udata);
+    GtGamesContainerChildPrivate* priv = gt_games_container_child_get_instance_private(self);
+    GtGame* game = GT_GAME(source);
+
+    gtk_stack_set_visible_child_name(GTK_STACK(priv->cover_stack),
+        gt_game_get_updating(game) ? "load-spinner" : "cover");
 }
 
 static void
@@ -85,10 +149,9 @@ get_property (GObject*    obj,
 }
 
 static void
-set_property(GObject*      obj,
-             guint         prop,
-             const GValue* val,
-             GParamSpec*   pspec)
+set_property(GObject* obj,
+    guint prop, const GValue* val,
+    GParamSpec* pspec)
 {
     GtGamesContainerChild* self = GT_GAMES_CONTAINER_CHILD(obj);
     GtGamesContainerChildPrivate* priv = gt_games_container_child_get_instance_private(self);
@@ -114,9 +177,15 @@ constructed(GObject* obj)
                            priv->name_label, "label",
                            G_BINDING_DEFAULT | G_BINDING_SYNC_CREATE);
     g_object_bind_property(priv->game, "preview",
-                           priv->preview_image, "pixbuf",
+                           priv->cover_image, "pixbuf",
                            G_BINDING_DEFAULT | G_BINDING_SYNC_CREATE);
 
+    g_signal_connect(priv->game, "notify::viewers", G_CALLBACK(update_viewers_cb), self);
+    g_signal_connect_object(priv->game, "notify::updating", G_CALLBACK(updating_cb), self, 0);
+
+    update_viewers_cb(G_OBJECT(priv->game), NULL, self);
+    updating_cb(G_OBJECT(priv->game), NULL, self);
+
     G_OBJECT_CLASS(gt_games_container_child_parent_class)->constructed(obj);
 }
 
@@ -144,10 +213,12 @@ gt_games_container_child_class_init(GtGamesContainerChildClass* klass)
                                                 "/com/vinszent/GnomeTwitch/ui/gt-games-container-child.ui");
     gtk_widget_class_bind_template_callback(GTK_WIDGET_CLASS(klass), motion_enter_cb);
     gtk_widget_class_bind_template_callback(GTK_WIDGET_CLASS(klass), motion_leave_cb);
-    gtk_widget_class_bind_template_child_private(GTK_WIDGET_CLASS(klass), GtGamesContainerChild, preview_image);
-    gtk_widget_class_bind_template_child_private(GTK_WIDGET_CLASS(klass), GtGamesContainerChild, middle_revealer);
+    gtk_widget_class_bind_template_child_private(GTK_WIDGET_CLASS(klass), GtGamesContainerChild, cover_image);
+    gtk_widget_class_bind_template_child_private(GTK_WIDGET_CLASS(klass), GtGamesContainerChild, cover_overlay_revealer);
     gtk_widget_class_bind_template_child_private(GTK_WIDGET_CLASS(klass), GtGamesContainerChild, name_label);
-    gtk_widget_class_bind_template_child_private(GTK_WIDGET_CLASS(klass), GtGamesContainerChild, event_box);
+    gtk_widget_class_bind_template_child_private(GTK_WIDGET_CLASS(klass), GtGamesContainerChild, cover_stack);
+    gtk_widget_class_bind_template_child_private(GTK_WIDGET_CLASS(klass), GtGamesContainerChild, viewers_label);
+    gtk_widget_class_bind_template_child_private(GTK_WIDGET_CLASS(klass), GtGamesContainerChild, viewers_image);
 }
 
 static void
@@ -158,10 +229,28 @@ gt_games_container_child_init(GtGamesContainerChild* self)
     gtk_widget_init_template(GTK_WIDGET(self));
 }
 
+GtGamesContainerChild*
+gt_games_container_child_new(GtGame* game)
+{
+    return g_object_new(GT_TYPE_GAMES_VIEW_CHILD,
+        "game", game, NULL);
+}
+
 void
 gt_games_container_child_hide_overlay(GtGamesContainerChild* self)
 {
     GtGamesContainerChildPrivate* priv = gt_games_container_child_get_instance_private(self);
 
-    gtk_revealer_set_reveal_child(GTK_REVEALER(priv->middle_revealer), FALSE);
+    gtk_revealer_set_reveal_child(GTK_REVEALER(priv->cover_overlay_revealer), FALSE);
+}
+
+
+GtGame*
+gt_games_container_child_get_game(GtGamesContainerChild* self)
+{
+    g_assert(GT_IS_GAMES_CONTAINER_CHILD(self));
+
+    GtGamesContainerChildPrivate* priv = gt_games_container_child_get_instance_private(self);
+
+    return priv->game;
 }
