@@ -1,3 +1,21 @@
+/*
+ *  This file is part of GNOME Twitch - 'Enjoy Twitch on your GNU/Linux desktop'
+ *  Copyright © 2017 Vincent Szolnoky <vinszent@vinszent.com>
+ *
+ *  GNOME Twitch is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ *
+ *  GNOME Twitch is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with GNOME Twitch. If not, see <http://www.gnu.org/licenses/>.
+ */
+
 #include "gt-player-header-bar.h"
 #include "gt-player.h"
 #include "gt-win.h"
@@ -18,6 +36,7 @@ typedef struct
     GtkWidget* back_button;
     GtkWidget* back_separator;
     GtkWidget* volume_button;
+    GtkWidget* stream_quality_box;
 
     GMenu* hamburger_menu;
 
@@ -26,10 +45,6 @@ typedef struct
     GtkWidget* show_chat_button;
 
     GtkAdjustment* chat_view_opacity_adjustment;
-    GtkAdjustment* chat_view_width_adjustment;
-    GtkAdjustment* chat_view_height_adjustment;
-    GtkAdjustment* chat_view_x_adjustment;
-    GtkAdjustment* chat_view_y_adjustment;
 } GtPlayerHeaderBarPrivate;
 
 G_DEFINE_TYPE_WITH_PRIVATE(GtPlayerHeaderBar, gt_player_header_bar, GTK_TYPE_HEADER_BAR)
@@ -104,8 +119,7 @@ mute_volume_cb(GtkWidget* button,
 
 static void
 player_channel_set_cb(GObject* source,
-                      GParamSpec* spec,
-                      gpointer udata)
+    GParamSpec* spec, gpointer udata)
 {
     GtPlayerHeaderBar* self = GT_PLAYER_HEADER_BAR(udata);
     GtPlayerHeaderBarPrivate* priv = gt_player_header_bar_get_instance_private(self);
@@ -114,7 +128,9 @@ player_channel_set_cb(GObject* source,
     gchar* status;
     GtChannel* chan;
 
-    g_object_get(win->player, "channel", &chan, NULL);
+    g_assert(GT_IS_WIN(win));
+
+    chan = gt_player_get_channel(win->player);
 
     if (chan)
     {
@@ -126,21 +142,56 @@ player_channel_set_cb(GObject* source,
         gtk_label_set_label(GTK_LABEL(priv->status_label), status);
         gtk_label_set_label(GTK_LABEL(priv->name_label), name);
 
-        g_object_unref(chan);
         g_free(name);
         g_free(status);
     }
 }
 
-static gboolean
-chat_pos_upper_transformer(GBinding* binding,
-                           const GValue* from,
-                           GValue* to,
-                           gpointer udata)
+static void
+player_playing_cb(GObject* source,
+    GParamSpec* pspec, gpointer udata)
 {
-    g_value_set_double(to, 1 - g_value_get_double(from));
+    g_assert(GT_IS_PLAYER_HEADER_BAR(udata));
 
-    return TRUE;
+    GtPlayerHeaderBar* self = GT_PLAYER_HEADER_BAR(udata);
+    GtPlayerHeaderBarPrivate* priv = gt_player_header_bar_get_instance_private(self);
+    GList* stream_qualities = NULL;
+    GtWin* win = GT_WIN_TOPLEVEL(self);
+
+    g_assert(GT_IS_WIN(win));
+
+    if (!gt_player_is_playing(win->player)) return;
+
+    stream_qualities = gt_player_get_available_stream_qualities(win->player);
+
+    utils_container_clear(GTK_CONTAINER(priv->stream_quality_box));
+
+    for (GList* l = stream_qualities; l != NULL; l = l->next)
+    {
+        GtkWidget* button;
+        GtTwitchStreamData* stream_data;
+        GVariant* target_variant;
+        g_autofree gchar* quality_text;
+
+        button = gtk_model_button_new();
+
+        stream_data = l->data;
+
+        g_assert_nonnull(stream_data);
+
+        target_variant = g_variant_new_string(stream_data->quality);
+
+        quality_text = utils_str_capitalise(stream_data->quality);
+
+        g_object_set(button,
+            "visible", TRUE,
+            "action-name", "player.set_stream_quality",
+            "action-target", target_variant,
+            "text", g_dgettext("gnome-twitch", quality_text),
+            NULL);
+
+        gtk_container_add(GTK_CONTAINER(priv->stream_quality_box), button);
+    }
 }
 
 static void
@@ -202,28 +253,6 @@ realise_cb(GtkWidget* widget,
     g_object_bind_property(win->player, "volume",
                            priv->volume_button, "value",
                            G_BINDING_BIDIRECTIONAL | G_BINDING_SYNC_CREATE | G_BINDING_BIDIRECTIONAL);
-    g_object_bind_property(win->player, "chat-width",
-                           priv->chat_view_width_adjustment, "value",
-                           G_BINDING_BIDIRECTIONAL | G_BINDING_SYNC_CREATE);
-    g_object_bind_property(win->player, "chat-height",
-                           priv->chat_view_height_adjustment, "value",
-                           G_BINDING_BIDIRECTIONAL | G_BINDING_SYNC_CREATE);
-    g_object_bind_property(win->player, "chat-x",
-                           priv->chat_view_x_adjustment, "value",
-                           G_BINDING_BIDIRECTIONAL | G_BINDING_SYNC_CREATE);
-    g_object_bind_property(win->player, "chat-y",
-                           priv->chat_view_y_adjustment, "value",
-                           G_BINDING_BIDIRECTIONAL | G_BINDING_SYNC_CREATE);
-    g_object_bind_property_full(win->player, "chat-width",
-                                priv->chat_view_x_adjustment, "upper",
-                                G_BINDING_DEFAULT | G_BINDING_SYNC_CREATE,
-                                (GBindingTransformFunc) chat_pos_upper_transformer,
-                                NULL, NULL, NULL);
-    g_object_bind_property_full(win->player, "chat-height",
-                                priv->chat_view_y_adjustment, "upper",
-                                G_BINDING_DEFAULT | G_BINDING_SYNC_CREATE,
-                                (GBindingTransformFunc) chat_pos_upper_transformer,
-                                NULL, NULL, NULL);
     g_object_bind_property(priv->chat_view_opacity_adjustment, "value",
                            win->player, "chat-opacity",
                            G_BINDING_BIDIRECTIONAL);
@@ -233,6 +262,7 @@ realise_cb(GtkWidget* widget,
 
     g_signal_connect(win->player, "notify::chat-visible", G_CALLBACK(chat_visible_cb), self);
     g_signal_connect(win->player, "notify::channel", G_CALLBACK(player_channel_set_cb), self);
+    g_signal_connect(win->player, "notify::playing", G_CALLBACK(player_playing_cb), self);
     g_signal_connect(win, "notify::fullscreen", G_CALLBACK(fullscreen_cb), self);
 
     player_channel_set_cb(NULL, NULL, self);
@@ -256,10 +286,6 @@ gt_player_header_bar_class_init(GtPlayerHeaderBarClass* klass)
     gtk_widget_class_bind_template_child_private(GTK_WIDGET_CLASS(klass), GtPlayerHeaderBar, unfullscreen_image);
     gtk_widget_class_bind_template_child_private(GTK_WIDGET_CLASS(klass), GtPlayerHeaderBar, volume_button);
     gtk_widget_class_bind_template_child_private(GTK_WIDGET_CLASS(klass), GtPlayerHeaderBar, chat_view_opacity_adjustment);
-    gtk_widget_class_bind_template_child_private(GTK_WIDGET_CLASS(klass), GtPlayerHeaderBar, chat_view_width_adjustment);
-    gtk_widget_class_bind_template_child_private(GTK_WIDGET_CLASS(klass), GtPlayerHeaderBar, chat_view_height_adjustment);
-    gtk_widget_class_bind_template_child_private(GTK_WIDGET_CLASS(klass), GtPlayerHeaderBar, chat_view_x_adjustment);
-    gtk_widget_class_bind_template_child_private(GTK_WIDGET_CLASS(klass), GtPlayerHeaderBar, chat_view_y_adjustment);
 //    gtk_widget_class_bind_template_child_private(GTK_WIDGET_CLASS(klass), GtPlayerHeaderBar, title_button);
     gtk_widget_class_bind_template_child_private(GTK_WIDGET_CLASS(klass), GtPlayerHeaderBar, status_label);
     gtk_widget_class_bind_template_child_private(GTK_WIDGET_CLASS(klass), GtPlayerHeaderBar, name_label);
@@ -268,6 +294,7 @@ gt_player_header_bar_class_init(GtPlayerHeaderBarClass* klass)
     gtk_widget_class_bind_template_child_private(GTK_WIDGET_CLASS(klass), GtPlayerHeaderBar, show_chat_button);
     gtk_widget_class_bind_template_child_private(GTK_WIDGET_CLASS(klass), GtPlayerHeaderBar, back_button);
     gtk_widget_class_bind_template_child_private(GTK_WIDGET_CLASS(klass), GtPlayerHeaderBar, back_separator);
+    gtk_widget_class_bind_template_child_private(GTK_WIDGET_CLASS(klass), GtPlayerHeaderBar, stream_quality_box);
 }
 
 static void
