@@ -1,9 +1,28 @@
+/*
+ *  This file is part of GNOME Twitch - 'Enjoy Twitch on your GNU/Linux desktop'
+ *  Copyright © 2017 Vincent Szolnoky <vinszent@vinszent.com>
+ *
+ *  GNOME Twitch is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ *
+ *  GNOME Twitch is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with GNOME Twitch. If not, see <http://www.gnu.org/licenses/>.
+ */
+
 #include <glib/gstdio.h>
 #include <glib.h>
 #include <stdlib.h>
 #include <string.h>
 #include "utils.h"
 #include "config.h"
+#include "gt-win.h"
 
 #define TAG "Utils"
 #include "gnome-twitch/gt-log.h"
@@ -36,18 +55,38 @@ utils_container_clear(GtkContainer* cont)
     }
 }
 
-gint64
-utils_timestamp_file(const gchar* filename)
+guint64
+utils_timestamp_file(const gchar* filename, GError** error)
 {
-    int ret;
-    GStatBuf file_stat;
+    RETURN_VAL_IF_FAIL(!utils_str_empty(filename), 0);
 
-    ret = g_stat(filename, &file_stat);
+    GError* err = NULL; /* NOTE: Doesn't need to be freed because we propagate it */
 
-    if (ret)
-        return 0;
+    if (g_file_test(filename, G_FILE_TEST_EXISTS))
+    {
+        GTimeVal time;
 
-    return file_stat.st_mtim.tv_sec;
+        g_autoptr(GFile) file = g_file_new_for_path(filename);
+
+        g_autoptr(GFileInfo) info = g_file_query_info(file,
+            G_FILE_ATTRIBUTE_TIME_MODIFIED, G_FILE_QUERY_INFO_NONE,
+            NULL, &err);
+
+        if (err)
+        {
+            WARNING("Could not timestamp file because: %s", err->message);
+
+            g_propagate_prefixed_error(error, err, "Could not timestamp file because: ");
+
+            return 0;
+        }
+
+        g_file_info_get_modification_time(info, &time);
+
+        return time.tv_sec;
+    }
+
+    RETURN_VAL_IF_REACHED(0);
 }
 
 gint64
@@ -74,71 +113,17 @@ utils_pixbuf_scale_simple(GdkPixbuf** pixbuf, gint width, gint height, GdkInterp
     *pixbuf = tmp;
 }
 
-static gint64
+gint64
 utils_http_full_date_to_timestamp(const char* string)
 {
-    gint64 ret;
-    SoupDate* tmp;
+    gint64 ret = G_MAXINT64;
+    g_autoptr(SoupDate) date = NULL;
 
-    tmp = soup_date_new_from_string(string);
-    ret = soup_date_to_time_t(tmp);
-    soup_date_free(tmp);
+    date = soup_date_new_from_string(string);
 
-    return ret;
-}
+    RETURN_VAL_IF_FAIL(date != NULL, ret);
 
-GdkPixbuf*
-utils_download_picture(SoupSession* soup, const gchar* url)
-{
-    SoupMessage* msg;
-    GdkPixbuf* ret = NULL;
-    GInputStream* input;
-    GError* err = NULL;
-
-    msg = soup_message_new("GET", url);
-    soup_message_headers_append(msg->request_headers, "Client-ID", CLIENT_ID);
-    input = soup_session_send(soup, msg, NULL, &err);
-
-    if (err)
-    {
-        WARNINGF("Error downloading picture, url=%s, code=%d, message=%s", url, err->code, err->message);
-        g_error_free(err);
-    }
-    else
-    {
-        ret = gdk_pixbuf_new_from_stream(input, NULL, NULL);
-
-        g_input_stream_close(input, NULL, NULL);
-    }
-
-    g_object_unref(msg);
-
-    return ret;
-}
-
-GdkPixbuf*
-utils_download_picture_if_newer(SoupSession* soup, const gchar* url, gint64 timestamp)
-{
-    SoupMessage* msg;
-    guint soup_status;
-    const gchar* last_modified;
-    GdkPixbuf* ret;
-
-    msg = soup_message_new(SOUP_METHOD_HEAD, url);
-    soup_message_headers_append(msg->request_headers, "Client-ID", CLIENT_ID);
-    soup_status = soup_session_send_message(soup, msg);
-
-    if (SOUP_STATUS_IS_SUCCESSFUL(soup_status) &&
-        (last_modified = soup_message_headers_get_one(msg->response_headers, "Last-Modified")) != NULL &&
-        utils_http_full_date_to_timestamp(last_modified) < timestamp)
-    {
-        g_info("{Utils} No new content at url '%s'", url);
-        ret = NULL;
-    }
-    else
-        ret = utils_download_picture(soup, url);
-
-    g_object_unref(msg);
+    ret = soup_date_to_time_t(date);
 
     return ret;
 }
@@ -203,13 +188,16 @@ utils_mouse_clicked_link_cb(GtkWidget* widget,
                             GdkEventButton* evt,
                             gpointer udata)
 {
-    GdkScreen* screen;
-
-    screen = gtk_widget_get_screen(widget);
-
     if (evt->button == 1 && evt->type == GDK_BUTTON_PRESS)
     {
-        gtk_show_uri(screen, (gchar*) udata, GDK_CURRENT_TIME, NULL);
+        GtWin* parent = GT_WIN_TOPLEVEL(widget);
+
+#if GTK_CHECK_VERSION(3, 22, 0)
+        gtk_show_uri_on_window(GTK_WINDOW(parent), (gchar*) udata, GDK_CURRENT_TIME, NULL);
+#else
+        gtk_show_uri(NULL, (gchar*) udata, GDK_CURRENT_TIME, NULL);
+#endif
+
     }
 
     return FALSE;
@@ -236,6 +224,16 @@ utils_str_empty(const gchar* str)
     return !(str && strlen(str) > 0);
 }
 
+gchar*
+utils_str_capitalise(const gchar* str)
+{
+    g_assert_false(utils_str_empty(str));
+
+    gchar* ret = g_strdup_printf("%c%s", g_ascii_toupper(*str), str+1);
+
+    return ret;
+}
+
 typedef struct
 {
     gpointer instance;
@@ -275,3 +273,87 @@ utils_signal_connect_oneshot(gpointer instance,
                           (GClosureNotify) g_free,
                           G_CONNECT_AFTER | G_CONNECT_SWAPPED);
 }
+
+void
+utils_signal_connect_oneshot_swapped(gpointer instance,
+    const gchar* signal, GCallback cb, gpointer udata)
+{
+
+    OneshotData* data = g_new(OneshotData, 1);
+
+    data->instance = instance;
+    data->cb = cb;
+    data->udata = udata;
+
+    g_signal_connect_swapped(instance, signal, cb, udata);
+
+    g_signal_connect_data(instance,
+                          signal,
+                          G_CALLBACK(oneshot_cb),
+                          data,
+                          (GClosureNotify) g_free,
+                          G_CONNECT_AFTER | G_CONNECT_SWAPPED);
+}
+
+inline void
+utils_refresh_cancellable(GCancellable** cancel)
+{
+    if (*cancel && !g_cancellable_is_cancelled(*cancel))
+        g_cancellable_cancel(*cancel);
+    g_clear_object(cancel);
+    *cancel = g_cancellable_new();
+}
+
+GWeakRef*
+utils_create_weak_ref(gpointer obj)
+{
+    GWeakRef* ref = g_malloc(sizeof(GWeakRef));
+
+    g_weak_ref_init(ref, obj);
+
+    return ref;
+}
+
+void
+utils_free_weak_ref(GWeakRef* ref)
+{
+    g_weak_ref_clear(ref);
+
+    g_clear_pointer(&ref, g_free);
+}
+
+GDateTime*
+utils_parse_time_iso_8601(const gchar* time, GError** error)
+{
+    GDateTime* ret = NULL;
+
+    gint year, month, day, hour, min, sec;
+
+    gint scanned = sscanf(time, "%d-%d-%dT%d:%d:%dZ", &year, &month, &day, &hour, &min, &sec);
+
+    if (scanned != 6)
+    {
+        g_set_error(error, GT_UTILS_ERROR, GT_UTILS_ERROR_PARSING_TIME,
+            "Unable to parse time from input '%s'", time);
+    }
+    else
+        ret = g_date_time_new_utc(year, month, day, hour, min, sec);
+
+    return ret;
+}
+
+GenericTaskData*
+generic_task_data_new()
+{
+    return g_slice_new0(GenericTaskData);
+}
+
+void
+generic_task_data_free(GenericTaskData* data)
+{
+    g_free(data->str_1);
+    g_free(data->str_2);
+    g_free(data->str_3);
+
+    g_slice_free(GenericTaskData, data);
+}
